﻿using System;
// ReSharper disable once RedundantUsingDirective
using System.Windows.Input;
// ReSharper disable once RedundantUsingDirective
using Windows.System;
// ReSharper disable once RedundantUsingDirective
using Windows.UI.Core;
// ReSharper disable once RedundantUsingDirective
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace UniversalSample.Commons
{
    /// <summary>
    ///     네비게이션 관련 헬퍼
    /// </summary>
    public class FrameHelper
    {
        private static FrameHelper _instance;

        /// <summary>
        ///     Frame object
        /// </summary>
        internal Frame _frame;

        /// <summary>
        ///     Singleton
        /// </summary>
        public static FrameHelper Instance
        {
            get { return _instance = _instance ?? new FrameHelper(); }
        }

        /// <summary>
        /// 네비게이팅 이벤트 from
        /// </summary>
        public event EventHandler<NavigatingCancelEventArgs> Navigating;

        /// <summary>
        /// OnNavigating
        /// </summary>
        /// <param name="s"></param>
        /// <param name="e"></param>
        protected virtual void OnNavigating(object s, NavigatingCancelEventArgs e)
        {
            EventHandler<NavigatingCancelEventArgs> handler = Navigating;
            if (handler != null) handler(s, e);
        }

        /// <summary>
        /// 네비게이티드 이벤트 to
        /// </summary>
        public event EventHandler<NavigationEventArgs> Navigated;

        /// <summary>
        /// OnNavigated
        /// </summary>
        /// <param name="s"></param>
        /// <param name="e"></param>
        protected virtual void OnNavigated(object s, NavigationEventArgs e)
        {
            EventHandler<NavigationEventArgs> handler = Navigated;
            if (handler != null) handler(s, e);
        }

        /// <summary>
        ///     RegisterFrame
        /// </summary>
        /// <param name="frame"></param>
        public void RegisterFrame(Frame frame)
        {
            _frame = frame;
            if (_frame == null) return;

            _frame.Navigated -= Frame_Navigated;
            _frame.Navigating -= Frame_Navigating;

            _frame.Navigated += Frame_Navigated;
            _frame.Navigating += Frame_Navigating;

#if WINDOWS_PHONE_APP
            Windows.Phone.UI.Input.HardwareButtons.BackPressed -= HardwareButtons_BackPressed;
            Windows.Phone.UI.Input.HardwareButtons.BackPressed += HardwareButtons_BackPressed;
#else
            // Listen to the window directly so focus isn't required
            Window.Current.CoreWindow.Dispatcher.AcceleratorKeyActivated +=
                CoreDispatcher_AcceleratorKeyActivated;
            Window.Current.CoreWindow.PointerPressed +=
                CoreWindow_PointerPressed;
#endif

        }

#if WINDOWS_PHONE_APP
        /// <summary>
        /// Invoked when the hardware back button is pressed. For Windows Phone only.
        /// </summary>
        /// <param name="sender">Instance that triggered the event.</param>
        /// <param name="e">Event data describing the conditions that led to the event.</param>
        private void HardwareButtons_BackPressed(object sender, Windows.Phone.UI.Input.BackPressedEventArgs e)
        {
            if (CanGoBack())
            {
                e.Handled = true;
                GoBack();
            }
        }
#else
        /// <summary>
        /// Invoked on every keystroke, including system keys such as Alt key combinations, when
        /// this page is active and occupies the entire window.  Used to detect keyboard navigation
        /// between pages even when the page itself doesn't have focus.
        /// </summary>
        /// <param name="sender">Instance that triggered the event.</param>
        /// <param name="e">Event data describing the conditions that led to the event.</param>
        private void CoreDispatcher_AcceleratorKeyActivated(CoreDispatcher sender,
            AcceleratorKeyEventArgs e)
        {
            var virtualKey = e.VirtualKey;

            // Only investigate further when Left, Right, or the dedicated Previous or Next keys
            // are pressed
            if ((e.EventType == CoreAcceleratorKeyEventType.SystemKeyDown ||
                e.EventType == CoreAcceleratorKeyEventType.KeyDown) &&
                (virtualKey == VirtualKey.Left || virtualKey == VirtualKey.Right ||
                (int)virtualKey == 166 || (int)virtualKey == 167))
            {
                var coreWindow = Window.Current.CoreWindow;
                const CoreVirtualKeyStates downState = CoreVirtualKeyStates.Down;
                bool menuKey = (coreWindow.GetKeyState(VirtualKey.Menu) & downState) == downState;
                bool controlKey = (coreWindow.GetKeyState(VirtualKey.Control) & downState) == downState;
                bool shiftKey = (coreWindow.GetKeyState(VirtualKey.Shift) & downState) == downState;
                bool noModifiers = !menuKey && !controlKey && !shiftKey;
                bool onlyAlt = menuKey && !controlKey && !shiftKey;

                if (((int)virtualKey == 166 && noModifiers) ||
                    (virtualKey == VirtualKey.Left && onlyAlt))
                {
                    // When the previous key or Alt+Left are pressed navigate back
                    e.Handled = true;
                    GoBack();
                }
                else if (((int)virtualKey == 167 && noModifiers) ||
                    (virtualKey == VirtualKey.Right && onlyAlt))
                {
                    // When the next key or Alt+Right are pressed navigate forward
                    //e.Handled = true;
                    //this.GoForwardCommand.Execute(null);
                }
            }
        }

        /// <summary>
        /// Invoked on every mouse click, touch screen tap, or equivalent interaction when this
        /// page is active and occupies the entire window.  Used to detect browser-style next and
        /// previous mouse button clicks to navigate between pages.
        /// </summary>
        /// <param name="sender">Instance that triggered the event.</param>
        /// <param name="e">Event data describing the conditions that led to the event.</param>
        private void CoreWindow_PointerPressed(CoreWindow sender,
            PointerEventArgs e)
        {
            var properties = e.CurrentPoint.Properties;

            // Ignore button chords with the left, right, and middle buttons
            if (properties.IsLeftButtonPressed || properties.IsRightButtonPressed ||
                properties.IsMiddleButtonPressed) return;

            // If back or foward are pressed (but not both) navigate appropriately
            bool backPressed = properties.IsXButton1Pressed;
            bool forwardPressed = properties.IsXButton2Pressed;
            if (backPressed ^ forwardPressed)
            {
                e.Handled = true;
                if (backPressed) GoBack();
                //if (forwardPressed) this.GoForwardCommand.Execute(null);
            }
        }
#endif

        /// <summary>
        ///     Navigation
        /// </summary>
        /// <param name="navigationType"></param>
        /// <param name="navigationParameter"></param>
        /// <returns></returns>
        public bool Navigation(Type navigationType, object navigationParameter)
        {
            bool returnValue;
            if (_frame == null) return false;

            returnValue = _frame.Navigate(navigationType, navigationParameter);

            return returnValue;
        }

        /// <summary>
        ///     GoBack
        /// </summary>
        /// <returns></returns>
        public bool GoBack()
        {
            if (_frame.CanGoBack)
            {
                _frame.GoBack();
            }

            return true;
        }

        /// <summary>
        ///     Navigation From
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Frame_Navigating(object sender, NavigatingCancelEventArgs e)
        {
            OnNavigating(sender, e);
        }

        /// <summary>
        ///     Navigation To
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Frame_Navigated(object sender, NavigationEventArgs e)
        {
            OnNavigated(sender, e);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool CanGoBack()
        {
            if (_frame == null) return false;
            return _frame.CanGoBack;
        }
    }
}