﻿using System;
using System.Collections.Generic;
using Windows.ApplicationModel;
using Windows.Storage;
using Windows.Storage.FileProperties;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;
using UniversalSample.Commons;
using UniversalSample.Design;
using UniversalSample.Models;
using UniversalSample.Views;

namespace UniversalSample.ViewModels
{
    public class MainPageVM : ViewModelBase
    {
        private IList<ClassM> _classList;
        private RelayCommand _itemClickCommand;
        private IList<PersonM> _people;
        private RelayCommand _openImageCommand;
        private ImageSource _selectedImageSource;
        private string _testText;

        public MainPageVM()
        {
            Title = "Kaki education center";

            ClassList = new List<ClassM>();
            People = new List<PersonM>();

            if (DesignMode.DesignModeEnabled)
            {
                //디자인 타임 데이터
                ClassList = DesignDatas.GetClassList();
                People = DesignDatas.GetPeople();
                
            }
        }

        public IList<ClassM> ClassList
        {
            get { return _classList; }
            set
            {
                _classList = value;
                OnPropertyChanged();
            }
        }

        public IList<PersonM> People
        {
            get { return _people; }
            set
            {
                _people = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        ///     ItemClick Command
        /// </summary>
        public RelayCommand ItemClickCommand
        {
            get
            {
                return _itemClickCommand = _itemClickCommand ?? new RelayCommand(
                    args =>
                    {
                        if (!(args is ItemClickEventArgs)) return;

                        var item = ((ItemClickEventArgs) args).ClickedItem as ClassM;
                        if (item == null) return;

                        FrameHelper.Instance.Navigation(typeof (ClassDetailPage), item);
                    });
            }
        }

        /// <summary>
        /// Selected Bitmap Image
        /// </summary>
        public ImageSource SelectedImageSource
        {
            get { return _selectedImageSource; }
            set { _selectedImageSource = value; OnPropertyChanged(); }
        }

        public string TestText
        {
            get { return _testText; }
            set { _testText = value; OnPropertyChanged();}
        }

        /// <summary>
        /// Open Image Command
        /// </summary>
        public RelayCommand OpenImageCommand
        {
            get { return _openImageCommand = _openImageCommand ?? new RelayCommand(async args =>
            {
                var openPicker = new FileOpenPicker();
                openPicker.ViewMode = PickerViewMode.Thumbnail;
                openPicker.SuggestedStartLocation = PickerLocationId.PicturesLibrary;
                openPicker.FileTypeFilter.Add(".jpg");
                openPicker.FileTypeFilter.Add(".jpeg");
                openPicker.FileTypeFilter.Add(".png");
                TestText = "FileOpen Start";
                StorageFile file = await StorageHelper.Instance.PickSingleFileAsync(openPicker);
                if (file != null)
                {
                    var bitmapImage = new BitmapImage();
                    using (IRandomAccessStream fileStream = await file.OpenAsync(FileAccessMode.Read))
                    {
                        // Set the image source to the selected bitmap
                        ImageProperties property = await file.Properties.GetImagePropertiesAsync();
                        bitmapImage.DecodePixelHeight = (int)property.Height;
                        bitmapImage.DecodePixelWidth = (int)property.Width;
                        await bitmapImage.SetSourceAsync(fileStream);
                    }
                    SelectedImageSource = bitmapImage;
                    TestText = "FileOpen End";
                }
            }); }
        }

        protected override void OnNavigated(NavigationEventArgs e)
        {
            //네비게이션이 완료 된 후 실행되는 부분, 지금은 디자인 타임과 동일한 데이터를 보여준다.
            ClassList = DesignDatas.GetClassList();
            People = DesignDatas.GetPeople();
        }
    }
}